using System.Net.Http;
using System.Reflection.Metadata;
using System.Transactions;
using System.Security.Permissions;
using System.Net;
using System.Net.Cache;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Enum.CourseEnum;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.DBRequest;
using System.Linq;
using WingInterfaceLibrary.Result.Courses;
using WingInterfaceLibrary.Request.Payment;
using WingServerCommon.Log;
using WingInterfaceLibrary.Notifications.Consultation;
using WingInterfaceLibrary.Internal.Request;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Notifications.Education;
using WingInterfaceLibrary.Request;
using System;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Request.User;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using System.IO;
using Newtonsoft.Json;

namespace WingEducationService.Service
{
    /// <summary>
    /// 课程服务服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {
        /// <summary>
        /// 申请课程
        /// </summary>
        /// <param name="request">申请课程请求</param>
        /// <returns>成功或者失败</returns>
        /// <errorCodes></errorCodes>
        public async Task<bool> ApplyCourseAsync(ApplyCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            //用户权限验证
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                request.TeacherCode = userCode;
            }
            var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
            if (teacherDTO == null)
            {
                ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
            }
            // var creatorDTO = await _userDBService.FindUserByCodeAsync(userCode);
            // if (creatorDTO == null)
            // {
            //     ThrowRpcException(CustomerRpcCode.UserNotFind, "User no found");
            // }

            var startTime = request.StartTime; //0012705  多媒体和视频课 开始时间设置最小值
            if (request.CourseType == CourseTypeEnum.Multimedia)
            {
                startTime = DateTime.MinValue;
            }
            if (request.Price > 0 && request.Price < 0.01m)
            {
                ThrowRpcException(CustomerRpcCode.IncorrectPrice, "Incorrect Price");
            }
            var price = Math.Round(request.Price, 2);            
            var createCourseInfoDTO = new CourseInfoDBRequest()
            {
                Name = request.Name,
                CourseIntro = request.CourseIntro,
                TeacherCode = request.TeacherCode,
                Cover = request.Cover,
                StartTime = startTime,
                Duration = request.Duration * 60,
                CourseType = request.CourseType,
                AudienceType = request.AudienceType,
                CoursewareToken = request.CoursewareToken,
                Status = CourseStatusEnum.NoApproval,
                TeacherName = teacherDTO.FullName,
                CreatorCode = userCode,
                OrganizationCode = teacherDTO.OrganizationCode,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes,
                UserGroupCodes = request.UserGroupCodes,
                Price = price,
                IsAgentCourse = request.IsAgentCourse,
                //BindExams = request.BindExams,
                CourseAppearType = request.CourseAppearType,
                CourseAlbumCodes = request.CourseAlbumCodes,
                Assistants = new List<StudentInfoDTO>(),
                Experts = new List<StudentInfoDTO>(),
                CourseVideoCodes = request.CourseVideoCodes,
                CourseMaterialCodes = request.CourseMaterialCodes,
                IsSmallClass = request.IsSmallClass,
                IsJustVideoCourse = request.IsJustVideoCourse
            };
            if (createCourseInfoDTO.Duration <= 0)
            {
                createCourseInfoDTO.Duration = 7200;
            }
            if (request.Assistants?.Count > 0)
            {
                foreach (var item in request.Assistants)
                {
                    if (string.IsNullOrWhiteSpace(item) || createCourseInfoDTO.Assistants.Any(a => a.Code == item))
                    {
                        continue;
                    }
                    var assistant = new StudentInfoDTO()
                    {
                        Code = item,
                        SignCourseStatus = StudentCourseStatusEnum.SignUp
                    };
                    createCourseInfoDTO.Assistants.Add(assistant);
                }
            }
            if (request.Experts?.Count > 0)
            {
                foreach (var item in request.Experts)
                {
                    if (string.IsNullOrWhiteSpace(item) || createCourseInfoDTO.Assistants.Any(a => a.Code == item) || createCourseInfoDTO.Experts.Any(a => a.Code == item))
                    {
                        continue;
                    }
                    var expert = new StudentInfoDTO()
                    {
                        Code = item,
                        SignCourseStatus = StudentCourseStatusEnum.SignUp
                    };
                    createCourseInfoDTO.Experts.Add(expert);
                }
            }
            var courseCode = await _courseDBService.CreateCourseAsync(new CreateCourseDBRequest() { Data = createCourseInfoDTO });

            if (!string.IsNullOrEmpty(courseCode) && teacherDTO.UserFeatureCodes?.Count > 0 && teacherDTO.UserFeatureCodes.Exists(c => c == "CertifiedInstructor"))
            {
                //如果是认证讲师，则直接审核
                var defaultToken = await _authenticationService.GetServerDefaultTokenAsync();
                var approvedReq = new WingInterfaceLibrary.Request.Management.AdminApprovalCourseRequest()
                {
                    Code = courseCode,
                    Status = CourseStatusEnum.Approved,
                    Token = defaultToken
                };
                var res = await _managementService.ApprovalCourseAsync(approvedReq);
                if (!res)
                {
                    Logger.WriteLineWarn($"EducationService ApplyCourseAsync ApprovalCourseAsync failed, courseCode: {courseCode}");
                }
            }
            if (!string.IsNullOrWhiteSpace(courseCode) && createCourseInfoDTO.Price > 0)
            {
                var createProductRequest = new CreateProductDBRequest
                {
                    Name = createCourseInfoDTO.Name,
                    HeadImage = createCourseInfoDTO.Cover,
                    Price = decimal.ToDouble(createCourseInfoDTO.Price),
                    Status = ProductStatusEnum.SoldOut,
                    ProductType = ProductTypeEnum.Course,
                    BelongerCode = createCourseInfoDTO.TeacherCode,
                    RelationInfo = courseCode,
                };
                await _paymentDBService.CreateProductAsync(createProductRequest);
            }
            return !string.IsNullOrWhiteSpace(courseCode);
        }

        /// <summary>
        /// 适配最优cdn地址
        /// </summary>
        /// <param name="requestUrl">源文件地址</param>
        /// <returns>cdn地址</returns>
        private string GetAdapterCDN(string requestUrl)
        {
            if (!string.IsNullOrEmpty(requestUrl))
            {
                //flyinsono-bj-1300984704|ap-beijing;flyinsono-hk-1300984704|ap-hongkong;flyinsono-fra-1300984704|eu-frankfurt
                //如果是cdn则使用新的，如果是源站，则直接返回
                var tempList = _otherNodes.Split(";");
                var sourceNodeList = new List<string>();
                foreach(var nodeInfo in tempList)
                {
                    var sourceNode = nodeInfo.Split("|")[0];
                    sourceNodeList.Add(sourceNode);
                }

                if (sourceNodeList?.Count > 0 && !string.IsNullOrEmpty(_vodHostNode))
                {
                    var isReturn = false;
                    foreach(var item in sourceNodeList)
                    {
                        if (requestUrl.Contains(item) || requestUrl.Contains(_vodHostNode))
                        {
                            isReturn = true;
                        }
                    }
                    if (isReturn)
                    {
                        return requestUrl;
                    }
                }
                var fileName = Path.GetFileName(requestUrl);
                var newUrl = _curCDNHost + "/" + fileName;
                return newUrl;
            }
            return "";
        }

        /// <summary>
        /// 通用课程列表查询接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CourseInfoDetailDTO>> FindCoursePagesAsync(FindCoursePagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (request.StartTime > DateTime.MinValue && request.EndTime > DateTime.MinValue)
            {
                if (request.StartTime > request.EndTime)
                {
                    ThrowRpcException((int)CustomerRpcCode.StartTimeGreaterThanEndTime, "StartTime Greater Than EndTime error");
                }
            }
            var dbRequest = new FindCoursePagesDBRequest()
            {
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                Keyword = request.Keyword,
                TeacherCode = request.TeacherCode,
                UserCode = userCode,
                StartTime = request.StartTime,
                EndTime = request.EndTime,
                CourseLabels = request.CourseLabels,
                CourseApprovalStatus = request.CourseApprovalStatus,
                ParticipantCode = request.ParticipantCode,
                QueryCourseType = request.QueryCourseType,
                LanguageCode = request.LanguageCode,
                JustPastVideo = request.JustPastVideo
            };
            var result = await _courseDBService.FindCoursePagesAsync(dbRequest);
            var coursePages = new PageResult<CourseInfoDetailDTO>();
            if (result == null)
            {
                return coursePages;
            }
            coursePages.TotalCount = result.TotalCount;
            coursePages.PageSize = result.PageSize;
            coursePages.PageIndex = result.PageIndex;

            if (result?.PageData?.Count > 0)
            {
                var courseLiveUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("Share", "CourseLiveUrl").Value;
                var courseShareUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("Share", "CourseShareUrl").Value;
                var courseCodeList = result.PageData.Select(s => s.Code).ToList();
                var signInfos = await _courseDBService.FindSignInListAsync(courseCodeList);
                signInfos = signInfos.Where(x => x.StudentCode == userCode).ToList();
                var allLabelList = await _educationDBService.GetAllCourseLabelDBAsync(new QueryCourseLabelDBRequest() { LanguageCode = request.LanguageCode }) ?? new List<CourseLabelDTO>();
                var list = new List<CourseInfoDetailDTO>();
                foreach (var pageDataItem in result.PageData)
                {
                    var courseLabels = allLabelList != null ? allLabelList.Where(w => pageDataItem.CourseLabelCodes.Contains(w.Code)).Select(f => new BaseLabelInfoDTO
                    {
                        Code = f.Code,
                        Name = f?.Name ?? string.Empty,
                        ParentCode = f.ParentCode
                    }).ToList() : new List<BaseLabelInfoDTO>();
                    var signInfo = signInfos.FirstOrDefault(w => w.CourseOrAlbumCode == pageDataItem.Code);
                    var detailInfo = new CourseInfoDetailDTO()
                    {
                        ActualStartTime = pageDataItem.ActualStartTime,
                        ActualEndTime = pageDataItem.ActualEndTime,
                        Code = pageDataItem.Code,
                        Name = pageDataItem.Name,
                        CourseIntro = pageDataItem.CourseIntro,
                        TeacherCode = pageDataItem.TeacherCode,
                        TeacherName = pageDataItem.TeacherName,
                        Cover = GetAdapterCDN(pageDataItem.Cover),
                        StartTime = pageDataItem.StartTime,
                        Duration = pageDataItem.Duration,
                        CourseType = pageDataItem.CourseType,
                        AudienceType = pageDataItem.AudienceType,
                        CoursewareToken = pageDataItem.CoursewareToken,
                        Status = pageDataItem.Status,
                        ViewRange = pageDataItem.ViewRange,
                        OrganizationCode = pageDataItem.OrganizationCode,
                        CaseLabels = new List<BaseLabelInfoDTO>(),
                        CourseLabels = courseLabels,
                        UserGroups = new List<BaseUserGroupDTO>(),
                        CaseLabelCodes = pageDataItem.CaseLabelCodes,
                        CourseLabelCodes = pageDataItem.CourseLabelCodes,
                        UserGroupCodes = pageDataItem.UserGroupCodes,
                        Price = pageDataItem.Price,
                        IsAgentCourse = pageDataItem.IsAgentCourse,
                        CreateTime = pageDataItem.CreateTime,
                        UpdateTime = pageDataItem.UpdateTime,
                        BindExams = pageDataItem.BindExams,
                        Experts = pageDataItem.Experts,
                        Assistants = pageDataItem.Assistants,
                        IsStick = pageDataItem.IsStick,
                        Sort = pageDataItem.Sort,
                        CourseAlbumCodes = pageDataItem.CourseAlbumCodes,
                        PlayCount = pageDataItem.PlayCount,
                        SignCourseStatus = StudentCourseStatusEnum.NoSignUp,
                        CourseLiveUrl = string.Format(courseLiveUrl, pageDataItem.Code),
                        CourseShareUrl = string.Format(courseShareUrl, pageDataItem.Code)
                    };
                    if (detailInfo.StartTime < DateTime.UtcNow && detailInfo.CourseType == CourseTypeEnum.LiveCourse)
                    {
                        detailInfo.SignCourseStatus = StudentCourseStatusEnum.Expired;
                    }
                    if (signInfo != null)
                    {
                        detailInfo.SignCourseStatus = signInfo.SignCourseStatus;
                        detailInfo.IsPay = signInfo.IsPay;
                    }
                    if (detailInfo.TeacherCode == userCode)
                    {
                        //课程是自己创建的 
                        detailInfo.SignCourseStatus = StudentCourseStatusEnum.MyCreated;
                        detailInfo.IsPay = false;
                    }
                    if (detailInfo.Experts?.Count > 0)
                    {
                        foreach (var expertItem in detailInfo.Experts)
                        {
                            if (expertItem.Code == userCode)
                            {
                                detailInfo.IsParticipant = true;
                            }
                            var userInfo = await _userDBService.FindUserDTOByCodeAsync(expertItem.Code, false);
                            expertItem.Name = !string.IsNullOrEmpty(userInfo?.FullName) ? userInfo?.FullName : userInfo?.UserName;
                        }
                    }
                    if (detailInfo.Assistants?.Count > 0)
                    {
                        foreach (var assistantItem in detailInfo.Assistants)
                        {
                            if (assistantItem.Code == userCode)
                            {
                                detailInfo.IsAssistant = true;
                            }
                            var userInfo = await _userDBService.FindUserDTOByCodeAsync(assistantItem.Code, false);
                            assistantItem.Name = !string.IsNullOrEmpty(userInfo?.FullName) ? userInfo?.FullName : userInfo?.UserName;
                        }
                    }
                    list.Add(detailInfo);
                }
                var sortList = list;
                if (!string.IsNullOrEmpty(request.TeacherCode))//我创建的课程
                {
                    sortList = list.OrderByDescending(c => c.CreateTime).ToList();
                }
                coursePages.PageData = sortList;
                {
                    //查询当前老师的专辑和班级
                    var albumList = await _courseDBService.GetCourseAlbumListByTeacherCodeDBAsync(userCode);
                    var userGroupList = await _educationDBService.GetUserGroupListsDBByCreatorCodeAsync(userCode);
                    if (albumList?.Count > 0 || userGroupList?.Count > 0)
                    {
                        foreach (var dataItem in result.PageData)
                        {
                            var pageInfo = coursePages.PageData.Find(c => c.Code == dataItem.Code);
                            pageInfo.CourseAlbums = new List<BaseCourseAlbumDTO>();
                            pageInfo.UserGroups = new List<BaseUserGroupDTO>();
                            if (dataItem?.CourseAlbumCodes?.Count > 0 && albumList?.Count > 0)
                            {
                                foreach (var albumCode in dataItem.CourseAlbumCodes)
                                {
                                    var albumInfo = new BaseCourseAlbumDTO()
                                    {
                                        Code = albumCode,
                                        Name = albumList.FirstOrDefault(c => c.Code == albumCode)?.Name ?? string.Empty
                                    };
                                    pageInfo.CourseAlbums.Add(albumInfo);
                                }
                            }

                            if (dataItem?.UserGroupCodes?.Count > 0 && userGroupList?.Count > 0)
                            {
                                foreach (var userGroupCode in dataItem.UserGroupCodes)
                                {
                                    var userGroupInfo = new BaseUserGroupDTO()
                                    {
                                        Code = userGroupCode,
                                        Name = userGroupList.FirstOrDefault(c => c.Code == userGroupCode)?.Name ?? string.Empty
                                    };
                                    pageInfo.UserGroups.Add(userGroupInfo);
                                }
                            }
                        }
                    }
                }
            }
            return coursePages;
        }

        /// <summary>
        /// 删除课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseByCodeAsync(DeleteCourseByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            //角色权限验证
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            if (existCourse.Status == CourseStatusEnum.RejectApproval || existCourse.Status == CourseStatusEnum.HasCancelled || existCourse.Status == CourseStatusEnum.NoApproval)//已经审核通过
            {
                return await _courseDBService.DeleteCourseByCodeAsync(new DeleteCourseByCodeDBRequest() { Code = request.Code });
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotDeleteForCourseStatus, "Not delete for course status");
            }
            return false;
        }

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseAsync(UpdateCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var code = request.Code;
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                request.TeacherCode = userCode;
            }
            var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
            if (teacherDTO == null)
            {
                ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            if (request.Price > 0 && request.Price < 0.01m)
            {
                ThrowRpcException(CustomerRpcCode.IncorrectPrice, "Incorrect Price");
            }
            var price = Math.Round(request.Price, 2);
            
            //检查状态
            if (existCourse.Status == CourseStatusEnum.RejectApproval || existCourse.Status == CourseStatusEnum.NoApproval)
            {
                var updateCourseInfoDTO = new CourseInfoDBRequest()
                {
                    Name = request.Name,
                    CourseIntro = request.CourseIntro,
                    TeacherCode = request.TeacherCode,
                    Cover = request.Cover,
                    StartTime = request.StartTime,
                    Duration = request.Duration * 60,
                    CourseType = request.CourseType,
                    AudienceType = request.AudienceType,
                    CoursewareToken = request.CoursewareToken,
                    TeacherName = teacherDTO.FullName,
                    CaseLabelCodes = request.CaseLabelCodes,
                    CourseLabelCodes = request.CourseLabelCodes,
                    UserGroupCodes = request.UserGroupCodes,
                    Price = price,
                    IsAgentCourse = request.IsAgentCourse,
                    //BindExams = request.BindExams,
                    CourseAppearType = request.CourseAppearType,
                    CourseAlbumCodes = request.CourseAlbumCodes,
                    Assistants = new List<StudentInfoDTO>(),
                    Experts = new List<StudentInfoDTO>(),
                    CourseVideoCodes = request.CourseVideoCodes,
                    CourseMaterialCodes = request.CourseMaterialCodes,
                    IsSmallClass = request.IsSmallClass,
                    IsJustVideoCourse = request.IsJustVideoCourse
                };
                if (updateCourseInfoDTO.Duration <= 0)
                {
                    updateCourseInfoDTO.Duration = 7200;
                }
                var signInfos = await _courseDBService.FindSignInListAsync(new List<string>() { code });               
                if (request.Assistants?.Count > 0)
                {
                    foreach (var item in request.Assistants)
                    {
                        if (string.IsNullOrWhiteSpace(item) || updateCourseInfoDTO.Assistants.Any(a => a.Code == item))
                        {
                            continue;
                        }
                        var assistant = new StudentInfoDTO()
                        {
                            Code = item,
                            SignCourseStatus = StudentCourseStatusEnum.SignUp
                        };
                        if (signInfos?.Count > 0 && signInfos.Any(c => c.StudentCode == assistant.Code))
                        {
                            ThrowRpcException(CustomerRpcCode.UserSignedFail, "User signed fail");
                        }
                        updateCourseInfoDTO.Assistants.Add(assistant);
                    }
                }
                if (request.Experts?.Count > 0)
                {
                    foreach (var item in request.Experts)
                    {
                        if (string.IsNullOrWhiteSpace(item) || updateCourseInfoDTO.Assistants.Any(a => a.Code == item) || updateCourseInfoDTO.Experts.Any(a => a.Code == item))
                        {
                            continue;
                        }
                        var expert = new StudentInfoDTO()
                        {
                            Code = item,
                            SignCourseStatus = StudentCourseStatusEnum.SignUp
                        };
                        if (signInfos?.Count > 0 && signInfos.Any(c => c.StudentCode == expert.Code))
                        {
                            ThrowRpcException(CustomerRpcCode.UserSignedFail, "User signed fail");
                        }
                        updateCourseInfoDTO.Experts.Add(expert);
                    }
                }
                if (existCourse.CourseAlbumCodes != null && existCourse.CourseAlbumCodes.Count > 0)
                {
                    foreach (var courseAlbumCode in existCourse.CourseAlbumCodes)
                    {
                        await _courseDBService.UpdateCourseAlbumDBAsync(new UpdateCourseAlbumDBRequest
                        {
                            CourseAlbumCode = courseAlbumCode,
                            RemoveCourseCodes = new List<string> { existCourse.Code }
                        });
                    }
                }
                if (request.CourseAlbumCodes != null && request.CourseAlbumCodes.Count > 0)
                {
                    foreach (var courseAlbumCode in request.CourseAlbumCodes)
                    {
                        await _courseDBService.UpdateCourseAlbumDBAsync(new UpdateCourseAlbumDBRequest
                        {
                            CourseAlbumCode = courseAlbumCode,
                            AddCourseCodes = new List<string> { existCourse.Code }
                        });
                    }
                    updateCourseInfoDTO.CourseAlbumCodes = request.CourseAlbumCodes;
                }
                var updateCourseResult = await _courseDBService.UpdateCourseAsync(new UpdateCourseDBRequest() { Data = updateCourseInfoDTO, Code = code });
                if (updateCourseResult && (existCourse.Price > 0 || updateCourseInfoDTO.Price > 0))
                {
                    var updateProductRequest = new UpdateProductByRelationDBRequest
                    {
                        RelationInfo = code,
                        Name = updateCourseInfoDTO.Name,
                        HeadImage = updateCourseInfoDTO.Cover,
                        Price = decimal.ToDouble(updateCourseInfoDTO.Price),
                        Status = (existCourse.Status == CourseStatusEnum.NoApproval || existCourse.Status == CourseStatusEnum.RejectApproval) ? ProductStatusEnum.SoldOut : ProductStatusEnum.Putaway,
                        ProductType = ProductTypeEnum.Course,
                        BelongerCode = updateCourseInfoDTO.TeacherCode,
                    };
                    await _paymentDBService.UpdateProductByRelationAsync(updateProductRequest);
                }
                return updateCourseResult;
            }
            else
            {
                ThrowRpcException(CustomerRpcCode.NotUpdateForCourseStatus, "Not update for course status");
            }
            return false;
        }

        /// <summary>
        /// 根据课程编码获取课程详细信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseInfoDetailDTO> FindCourseByCodeAsync(FindCourseByCodeRequest request)
        {
            var userCode = "";
            if (!string.IsNullOrWhiteSpace(request.Token))
            {
                userCode = await GetClientIdByTokenAsync(request.Token);
                if (string.IsNullOrEmpty(userCode))
                {
                    ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
                }
            }
            var code = request.Code;
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var languageCode = request.LanguageCode;
            if (string.IsNullOrWhiteSpace(languageCode))
            {
                languageCode = "en-US";
            }
            var resultData = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code, LanguageCode = languageCode });
            if (!string.IsNullOrWhiteSpace(userCode))
            {
                var signInfos = await _courseDBService.FindSignInListAsync(new List<string> { code });
                var signInfo = signInfos.FirstOrDefault(x => x.StudentCode == userCode);
                if (signInfo != null)
                {
                    resultData.SignCourseStatus = signInfo.SignCourseStatus;
                    resultData.IsPay = signInfo.IsPay;
                }
                else
                {
                    resultData.SignCourseStatus = StudentCourseStatusEnum.NoSignUp;
                }
                if (resultData.TeacherCode == userCode)
                {
                    //课程是自己创建的 
                    resultData.SignCourseStatus = StudentCourseStatusEnum.MyCreated;
                    resultData.IsPay = false;
                }
                if (resultData.Experts != null && resultData.Experts.Count > 0 && resultData.Experts.Select(x => x.Code).Contains(userCode))
                {
                    resultData.IsParticipant = true;
                }
                else if (resultData.Assistants != null && resultData.Assistants.Count > 0 && resultData.Assistants.Select(x => x.Code).Contains(userCode))
                {
                    resultData.IsAssistant = true;
                }
                if (resultData.Experts?.Count > 0)
                {
                    foreach (var expertItem in resultData.Experts)
                    {
                        var userInfo = await _userDBService.FindUserDTOByCodeAsync(expertItem.Code, false);
                        expertItem.Name = !string.IsNullOrEmpty(userInfo?.FullName) ? userInfo?.FullName : userInfo?.UserName;
                    }
                }
                if (resultData.Assistants?.Count > 0)
                {
                    foreach (var assistantItem in resultData.Assistants)
                    {
                        var userInfo = await _userDBService.FindUserDTOByCodeAsync(assistantItem.Code, false);
                        assistantItem.Name = !string.IsNullOrEmpty(userInfo?.FullName) ? userInfo?.FullName : userInfo?.UserName;
                    }
                }
            }
            if (resultData != null && resultData.CourseType == CourseTypeEnum.LiveCourse)
            {
                resultData.Duration = resultData.Duration / 60;
            }
            var courseLiveUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("Share", "CourseLiveUrl").Value;
            var courseShareUrl = EnvironmentConfigManager.GetParammeter<StringParameter>("Share", "CourseShareUrl").Value;
            resultData.CourseLiveUrl = string.Format(courseLiveUrl, code);
            resultData.CourseShareUrl = string.Format(courseShareUrl, code);
            resultData.Cover = GetAdapterCDN(resultData.Cover);
            return resultData;
        }

        /// <summary>
        /// 学生已报名课程列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CourseInfoDetailDTO>> FindMyCoursePagesAsync(FindMyCoursePagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            var dbRequest = new FindMyCoursePagesDBRequest();
            dbRequest.CourseAppearType = CourseAppearTypeEnum.Independent;
            dbRequest.QueryStatus = request.QueryStatus;
            dbRequest.StudentCode = userCode;
            dbRequest.Keyword = request.Keyword;
            dbRequest.PageIndex = request.PageIndex;
            dbRequest.PageSize = request.PageSize;
            var result = await _courseDBService.FindMyCoursePagesDBAsync(dbRequest);
            var coursePages = new PageResult<CourseInfoDetailDTO>();
            if (result == null)
            {
                return coursePages;
            }
            coursePages.TotalCount = 0;
            coursePages.PageSize = request.PageSize;
            coursePages.PageIndex = request.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                IList<CourseInfoDetailDTO> courseInfos = null;
                var courseList = result.PageData.Where(w => w.CourseAppearType == CourseAppearTypeEnum.Independent).Select(s => s.CourseOrAlbumCode).ToList();
                if (courseList != null && courseList.Count > 0)
                {
                    courseInfos = await _courseDBService.FindCoursesByCodesAsync(new FindCourseByCodesDBRequest
                    {
                        Codes = courseList
                    });
                    foreach(var item in courseInfos)
                    {
                        item.Cover = GetAdapterCDN(item.Cover);
                    }
                    coursePages.TotalCount = courseInfos.Count;
                    coursePages.PageData = courseInfos.OrderByDescending(c => c.StartTime).Skip(request.PageSize * (request.PageIndex - 1)).Take(request.PageSize).ToList();
                }
            }
            return coursePages;
        }

        /// <summary>
        /// 学生已报名专辑列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CourseAlbumDTO>> FindMyAlbumPagesAsync(FindMyCoursePagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            var dbRequest = new FindMyCoursePagesDBRequest();
            dbRequest.CourseAppearType = CourseAppearTypeEnum.Album;
            dbRequest.QueryStatus = request.QueryStatus;
            dbRequest.StudentCode = userCode;
            dbRequest.Keyword = request.Keyword;
            dbRequest.PageIndex = request.PageIndex;
            dbRequest.PageSize = request.PageSize;
            var result = await _courseDBService.FindMyCoursePagesDBAsync(dbRequest);
            var courseAlbumPages = new PageResult<CourseAlbumDTO>();
            if (result == null)
            {
                return courseAlbumPages;
            }
            courseAlbumPages.TotalCount = result.TotalCount;
            courseAlbumPages.PageSize = request.PageSize;
            courseAlbumPages.PageIndex = request.PageIndex;
            if (result.PageData != null && result.PageData.Count > 0)
            {
                IList<CourseAlbumDTO> albumInfos = null;
                var albumList = result.PageData.Where(w => w.CourseAppearType == CourseAppearTypeEnum.Album).Select(s => s.CourseOrAlbumCode).ToList();
                if (albumList != null && albumList.Count > 0)
                {
                    albumInfos = await _courseDBService.FindAllAlbumListByAlbumCodesDBAsync(albumList);
                    foreach(var item in albumInfos)
                    {
                        item.Cover = GetAdapterCDN(item.Cover);
                    }
                    courseAlbumPages.PageData = albumInfos.OrderByDescending(c => c.CreateTime).Skip(request.PageSize * (request.PageIndex - 1)).Take(request.PageSize).ToList();
                }
            }
            return courseAlbumPages;
        }

        /// <summary>
        /// 学生增加课程播放量
        /// </summary>
        /// <returns></returns>
        public async Task<bool> AddCoursePlayCountAsync(AddCoursePlayCountRequest request)
        {
            //  var userCode = await GetClientIdByTokenAsync(request.Token);
            // if (string.IsNullOrEmpty(userCode))
            // {
            //     ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            // }
            var code = request.Code;
            if (string.IsNullOrEmpty(code))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            var existCourse = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = code });
            //课程未找到
            if (existCourse == null)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "Course no found");
            }
            //视频课和多媒体课 执行播放量
            if (existCourse.CourseType == CourseTypeEnum.Multimedia || existCourse.CourseType == CourseTypeEnum.LiveCourse)
            {
                return await _courseDBService.AddCoursePlayCountDBAsync(new AddCoursePlayCountDBRequest()
                {
                    Code = code,
                    AddPlayCount = 1
                });
            }
            return false;
        }
        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddExaminationPaperAsync(AddExaminationPaperRequest request)
        {
            var dbRequest = new AddExaminationPaperDBRequest
            {
                Name = request.Name,
                TotalScore = request.TotalScore,
                QuestionList = request.QuestionList,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.AddExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteExaminationPaperAsync(DeleteExaminationPaperRequest request)
        {
            var dbRequest = new DeleteExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode
            };
            return await _educationDBService.DeleteExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateExaminationPaperAsync(UpdateExaminationPaperRequest request)
        {
            var dbRequest = new UpdateExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode,
                Name = request.Name,
                TotalScore = request.TotalScore,
                QuestionList = request.QuestionList,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.UpdateExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 查询试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ExaminationPaperDTO> GetExaminationPaperAsync(GetExaminationPaperRequest request)
        {
            var dbRequest = new GetExaminationPaperDBRequest
            {
                ExaminationPaperCode = request.ExaminationPaperCode
            };
            return await _educationDBService.GetExaminationPaperAsync(dbRequest);
        }

        /// <summary>
        /// 查询试卷分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<ExaminationPaperDTO>> GetExaminationPaperPagesAsync(GetExaminationPaperPagesRequest request)
        {
            var dbRequest = new GetExaminationPaperPagesDBRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                CaseLabelCodes = request.CaseLabelCodes,
                CourseLabelCodes = request.CourseLabelCodes
            };
            return await _educationDBService.GetExaminationPaperPagesAsync(dbRequest);
        }

        /// <summary>
        /// 修改素材名称
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> ChangeVideoNameAsync(ChangeVideoNameRequest request)
        {
            return await _courseDBService.ChangeVideoNameDBAsync(request.VideoCode, request.NewName);
        }

        /// <summary>
        /// 创建素材
        /// </summary>Vide
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateVideoAsync(SaveVideoRequest request)
        {
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            if (string.IsNullOrEmpty(userInfo.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            //Code验证
            if (string.IsNullOrEmpty(request.Name))
            {
                ThrowRpcException(CustomerRpcCode.VideoNameIsEmpty, "Video name is empty");
            }
            if (string.IsNullOrEmpty(request.VideoToken))
            {
                ThrowRpcException(CustomerRpcCode.VideoTokenIsEmpty, "Video token is empty");
            }
            if (request.Duration <= 0)
            {
                ThrowRpcException(CustomerRpcCode.CourseVideoDurationNotZero, "Video duration not zero");
            }
            if (string.IsNullOrEmpty(request.CreatorCode))
            {
                request.CreatorCode = userInfo.ClientId;
            }
            if (string.IsNullOrEmpty(request.CreatorName))
            {
                request.CreatorName = userInfo.AccountName;
            }
            request.Duration = request.Duration * 60;
            var result = await _courseDBService.CreateVideoDBAsync(request);
            return result;
        }

        /// <summary>
        /// 创建素材
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CreateVideosAsync(CreateVideosRequest request)
        {
            var userInfo = await _authenticationService.GetTokenAsync(new TokenRequest() { Token = request.Token }) ?? new TokenDTO();
            if (string.IsNullOrEmpty(userInfo.ClientId))
            {
                ThrowCustomerException(CustomerRpcCode.UsercodeIsEmpty, "User code is empty");
            }
            //Code验证
            if (request.VideoList == null || request.VideoList.Count <= 0)
            {
                ThrowRpcException(CustomerRpcCode.VideosIsEmpty, "Videos is empty");
            }
            var result = true;
            foreach (var item in request.VideoList)
            {
                if (string.IsNullOrEmpty(item.CreatorCode))
                {
                    item.CreatorCode = userInfo.ClientId;
                }
                if (string.IsNullOrEmpty(item.CreatorName))
                {
                    item.CreatorName = userInfo.AccountName;
                }
                item.Duration = item.Duration * 60;
                var res = await _courseDBService.CreateVideoDBAsync(item);
                result = result & res;
            }
            return result;
        }

        /// <summary>
        /// 查询课程文件分类列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideoFilesCourseTypeByPageAsync(FindVideoFilesCourseTypeByPageRequest request)
        {

            var userCode = await GetClientIdByTokenAsync(request.Token);
            List<UploadFileTypeEnum> fileTypes = new List<UploadFileTypeEnum>();
            if (request.CourseType == CourseTypeEnum.Multimedia && request.CourseFileCategoryEnum == QueryCourseFileCategoryEnum.CourseWare)
            {
                //多媒体课 课程源设置
                fileTypes.AddRange(new List<UploadFileTypeEnum>()
                {
                    UploadFileTypeEnum.MP4,
                    UploadFileTypeEnum.PPT,
                    UploadFileTypeEnum.PPTX,
                    UploadFileTypeEnum.PDF
                });
            }
            var result = await _courseDBService.FindVideoFilesCourseTypeByPageDBAsync(new FindVideoFilesCourseTypeByPageDBRequest()
            {
                Keyword = request.Keyword,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize,
                FileTypes = fileTypes,
                UserCode = userCode
            });
            return result;
        }


        /// <summary>
        /// 查询所有视频列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<VideoInfoDTO>> FindVideosPagesAsync(FindVideoPagesRequest request)
        {
            var result = await _courseDBService.FindVideosPagesDBAsync(request);
            return result;
        }

        /// <summary>
        /// 删除视频，同步删除课程管理视频code
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteVideoByCodeAsync(DeleteVideoRequest request)
        {
            //Code验证
            if (string.IsNullOrEmpty(request.Code))
            {
                ThrowRpcException(CustomerRpcCode.VideoCodeIsEmpty, "video code is empty");
            }
            var result = await _courseDBService.DeleteVideoByCodeDBAsync(request);
            return result;
        }

        /// <summary>
        /// 购买课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<BuyCourseResult> BuyCourseAsync(BuyCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var course = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = request.CourseCode });
            if (course == null || string.IsNullOrWhiteSpace(course.Code))
            {
                ThrowRpcException(CustomerRpcCode.CourseNoFound, "CourseNoFound");
            }
            if (course.Status == CourseStatusEnum.NoApproval || course.Status == CourseStatusEnum.RejectApproval)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotReviewed, "CourseNotReviewed");
            }
            if (course.Price <= 0)
            {
                ThrowRpcException(CustomerRpcCode.CourseNoPayRequired, "CourseNoPayRequired");
            }
            if (request.Quantity <= 0)
            {
                ThrowRpcException(CustomerRpcCode.CourseQuantityError, "CourseQuantityError");
            }
            if (course.Experts != null && course.Experts.Count > 0 && course.Experts.Select(x => x.Code).Contains(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.ParticipantCantBuy, "ParticipantCantBuy");
            }
            if (course.Assistants != null && course.Assistants.Count > 0 && course.Assistants.Select(x => x.Code).Contains(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.AssistantCantBuy, "AssistantCantBuy");
            }
            await _courseDBService.SignUpCourseDBAsync(new SignUpCourseDBRequest()
            {
                CourseOrAlbumCode = request.CourseCode,
                StudentCode = userCode,
                CourseAppearType = CourseAppearTypeEnum.Independent
            });
            var product = await _paymentDBService.GetProductByRelationInfoAsync(new GetProductByRelationInfoDBRequest { RelationInfo = course.Code });
            if (product == null || string.IsNullOrWhiteSpace(product.Code) || product.Status != ProductStatusEnum.Putaway)
            {
                ThrowRpcException(CustomerRpcCode.CourseNotReviewed, "CourseNotReviewed");
            }
            var orderAmount = decimal.ToDouble(course.Price * request.Quantity);
            var createOrderRequest = new CreateOrderRequest
            {
                OrderUserCode = userCode,
                ProductCode = product.Code,
                OrderTitle = course.Name,
                OrderType = OrderTypeEnum.General,
                OrderAmount = orderAmount,
            };
            var createOrderResult = await _paymentService.CreateOrderAsync(createOrderRequest);
            var orderCode = createOrderResult.OrderCode;
            if (string.IsNullOrWhiteSpace(orderCode))
            {
                ThrowRpcException(CustomerRpcCode.CreateCourseOrderFailed, "CreateCourseOrderFailed");
            }
            var prepayResult = await _paymentService.GoToPayAsync(new GoToPayRequest
            {
                OrderCode = orderCode,
                PayType = request.PayType,
            });
            var result = new BuyCourseResult
            {
                PaymentOrderCode = orderCode,
                PayUrl = prepayResult.PayUrl,
                OrderTitle = course.Name,
                OrderAmount = orderAmount
            };
            return result;
        }

        /// <summary>
        /// 支付结果回调
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> PaymentCallbackAsync(PaymentCallbackRequest request)
        {
            if (request.PayStatus == PayStatusEnum.Paid)
            {
                var paySuccessRequest = new PaySuccessDBRequest
                {
                    PaymentOrderCode = request.PaymentOrderCode,
                };
                var paySuccessResult = await _paymentDBService.PaySuccessAsync(paySuccessRequest);
                if (paySuccessResult.Success)
                {
                    if (!string.IsNullOrWhiteSpace(paySuccessResult.OrderUserCode))
                    {
                        //课程支付成功通知 
                        var paySuccessNotification = new CourcePaySuccessNotification
                        {
                            PaymentOrderCode = request.PaymentOrderCode,
                            OrderTitle = paySuccessResult.OrderTitle,
                            OrderAmount = paySuccessResult.OrderAmount,
                            PayTime = paySuccessResult.PayTime,
                        };
                        var broadcastNotificationRequest = new BroadcastNotificationRequest
                        {
                            ClientIds = new List<string> { paySuccessResult.OrderUserCode },
                            Message = paySuccessNotification,
                            JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(paySuccessNotification),
                            NotificationType = paySuccessNotification.NotificationType,
                            TransactionType = TransactionTypeEnum.Education,
                            RelevanceCode = paySuccessResult.RelationInfo,
                            ReceiverType = ApplicantTypeEnum.Client
                        };
                        await _notificationService.BroadcastMessageAsync(broadcastNotificationRequest);
                    }
                    return true;
                }
                else
                {
                    Logger.WriteLineWarn($"EducationService PaymentCallbackAsync failed, order code:{request.PaymentOrderCode}, pay status:{request.PayStatus.ToString()}");
                }
            }
            else
            {
                Logger.WriteLineWarn($"EducationService PaymentCallbackAsync, order code:{request.PaymentOrderCode}, pay status:{request.PayStatus.ToString()}");
            }
            return await Task.FromResult(false);
        }


        /// <summary>
        /// 报名课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SignUpCourseAsync(SignUpCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseOrAlbumCode = request.CourseOrAlbumCode;
            var courseAppearType = request.CourseAppearType;
            if (string.IsNullOrEmpty(courseOrAlbumCode))
            {
                if (courseAppearType == CourseAppearTypeEnum.Album)
                {
                    ThrowCustomerException(CustomerRpcCode.CourseAlbumCodeEmpty, "CourseAlbum code is empty");
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
                }
            }
            var code = await _courseDBService.SignUpCourseDBAsync(new SignUpCourseDBRequest()
            {
                CourseOrAlbumCode = courseOrAlbumCode,
                StudentCode = userCode,
                CourseAppearType = courseAppearType
            });
            if (!string.IsNullOrWhiteSpace(code))
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// 取消报名课程
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> CancelSignUpCourseAsync(CancelSignUpCourseRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var courseOrAlbumCode = request.CourseOrAlbumCode;
            if (string.IsNullOrEmpty(courseOrAlbumCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            return await _courseDBService.CancelSignUpCourseDBAsync(new CancelSignUpCourseDBRequest()
            {
                CourseOrAlbumCode = courseOrAlbumCode,
                StudentCode = userCode
            });
        }


        /// <summary>
        /// 学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddStudentInCourseUserGroupAsync(AddStudentInCourseUserGroupRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.UserGroupCode) && string.IsNullOrEmpty(request.ShortCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            //
            var dbRequest = new AddStudentInCourseUserGroupDBRequest();
            if (!string.IsNullOrEmpty(request.UserGroupCode))
            {
                var userGroupList = await _educationDBService.GetUserGroupListsDBByUserGroupCodesAsync(new List<string>() { request.UserGroupCode });
                if (userGroupList?.Count > 0)
                {
                    foreach (var item in userGroupList)
                    {
                        if (item.CreatorCode == userCode)
                        {
                            ThrowRpcException(CustomerRpcCode.ClassTeacherCodeNotAddedClass, "Class teacher not added class");
                        }
                    }
                }
                dbRequest.UserGroupCode = request.UserGroupCode;
            }
            else if (!string.IsNullOrEmpty(request.ShortCode))
            {
                var userGroup = await _educationDBService.GetUserGroupDBByShortCodeAsync(request.ShortCode);
                if (userGroup.CreatorCode == userCode)
                {
                    ThrowRpcException(CustomerRpcCode.ClassTeacherCodeNotAddedClass, "Class teacher not added class");
                }
                dbRequest.UserGroupCode = userGroup.Code;
            }
            dbRequest.StudentCodes = new List<string> { userCode };
            return await _courseDBService.AddStudentInCourseUserGroupDBAsync(dbRequest);
        }

        /// <summary>
        /// 学生退出班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> StudentLeaveCourseUserGroupAsync(StudentLeaveCourseUserGroupRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var dbRequest = new StudentLeaveCourseUserGroupDBRequest();
            dbRequest.UserGroupCode = request.UserGroupCode;
            dbRequest.StudentCodes = new List<string> { userCode };
            return await _courseDBService.StudentLeaveCourseUserGroupDBAsync(dbRequest);
        }

        /// <summary>
        /// 学习完成处理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> StudyCompletedAsync(StudyCompletedRequest request)
        {
            if (string.IsNullOrEmpty(request.CourseCode))
            {
                ThrowRpcException(CustomerRpcCode.CourseCodeEmpty, "Course code is empty");
            }
            if (string.IsNullOrWhiteSpace(request.StudentCode))
            {
                ThrowRpcException(CustomerRpcCode.UserCodeEmptyError, "UserCodeEmptyError");
            }
            var dbRequest = new StudyCompletedDBRequest();
            dbRequest.CourseCode = request.CourseCode;
            dbRequest.StudentCode = request.StudentCode;
            return await _courseDBService.StudyCompletedDBAsync(dbRequest);
        }

        /// <summary>
        /// 添加课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<string> AddCourseAlbumAsync(AddCourseAlbumRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.Name))
            {
                ThrowCustomerException(CustomerRpcCode.AlbumNameIsEmpty, "Album name is empty");
            }
            //用户权限验证
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                request.TeacherCode = userCode;
            }
            if (string.IsNullOrEmpty(request.TeacherName))
            {
                var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
                if (teacherDTO == null)
                {
                    ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
                }
                else
                {
                    request.TeacherName = teacherDTO.FullName;
                }
            }
            return await _courseDBService.AddCourseAlbumDBAsync(new AddCourseAlbumDBRequest
            {
                Data = new CourseAlbumDTO
                {
                    Name = request.Name,
                    CourseCodes = request.CourseCodes,
                    CourseLabelCodes = request.CourseLabelCodes,
                    Cover = request.Cover,
                    Price = request.Price,
                    TeacherCode = request.TeacherCode,
                    TeacherName = request.TeacherName,
                    Introduction = request.Introduction
                }
            });
        }

        /// <summary>
        /// 修改课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateCourseAlbumAsync(UpdateCourseAlbumRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(request.CourseAlbumCode))
            {
                ThrowCustomerException(CustomerRpcCode.AlbumCodeIsEmpty, "Album code is empty");
            }
            if (string.IsNullOrWhiteSpace(request.Name))
            {
                ThrowCustomerException(CustomerRpcCode.AlbumNameIsEmpty, "Album name is empty");
            }
            //用户权限验证
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                request.TeacherCode = userCode;
            }
            if (string.IsNullOrEmpty(request.TeacherName))
            {
                var teacherDTO = await _userDBService.FindUserByCodeAsync(request.TeacherCode);
                if (teacherDTO == null)
                {
                    ThrowRpcException(CustomerRpcCode.TeacherNoFound, "Teacher name no found");
                }
                else
                {
                    request.TeacherName = teacherDTO.FullName;
                }
            }
            var dbRequest = new UpdateCourseAlbumDBRequest()
            {
                CourseAlbumCode = request.CourseAlbumCode,
                Name = request.Name,
                TeacherCode = request.TeacherCode,
                TeacherName = request.TeacherName,
                Price = request.Price,
                AddCourseCodes = request.CourseCodes,
                Cover = request.Cover,
                AddCourseLabelCodes = request.CourseLabelCodes,
                Introduction = request.Introduction,
                IsClientSubmit = true
            };
            return await _courseDBService.UpdateCourseAlbumDBAsync(dbRequest);
        }


        /// <summary>
        /// 删除课程专辑
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> DeleteCourseAlbumAsync(DeleteCourseAlbumRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.CourseAlbumCode))
            {
                ThrowCustomerException(CustomerRpcCode.CodeIsEmpty, "CodeIsEmpty");
            }
            return await _courseDBService.DeleteCourseAlbumDBAsync(new DeleteCourseAlbumDBRequest
            {
                CourseAlbumCode = request.CourseAlbumCode
            });
        }

        /// <summary>
        /// 查询课程专辑分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<CourseCommonInfoDTO>> FindCourseAlbumPageAsync(FindCoursePagesRequest request)
        {
            var res = new PageResult<CourseCommonInfoDTO>()
            {
                PageIndex = request.PageIndex,
                PageSize = request.PageSize
            };
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var dataList = await _courseDBService.FindCourseAlbumPageDBAsync(new FindCourseAlbumPageDBRequest
            {
                Keyword = request.Keyword,
                CourseLabelCodes = request.CourseLabels,
                StartTime = request.StartTime,
                EndTime = request.EndTime,
                TeacherCode = request.TeacherCode,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize
            });
            if (dataList?.PageData?.Count > 0)
            {
                var allLabelList = await _educationDBService.GetAllCourseLabelDBAsync(new QueryCourseLabelDBRequest() { LanguageCode = request.LanguageCode }) ?? new List<CourseLabelDTO>();
                var codes = dataList?.PageData?.Select(x => x.Code).ToList();
                var signInfos = await _courseDBService.FindSignInListAsync(codes);
                signInfos = signInfos.Where(x => x.StudentCode == userCode).ToList();
                var list = new List<CourseCommonInfoDTO>();
                foreach (var item in dataList.PageData)
                {
                    var signInfo = signInfos.FirstOrDefault(x => x.CourseOrAlbumCode == item.Code);
                    var model = new CourseCommonInfoDTO()
                    {
                        Code = item.Code,
                        Name = item.Name,
                        Cover = item.Cover,
                        Introduction = item.Introduction,
                        CourseLabels = new List<string>(),
                        TeacherCode = item.TeacherCode,
                        TeacherName = item.TeacherName,
                        ViewRange = item.ViewRange,
                        Price = item.Price,
                        CourseType = CourseTypeEnum.Unknown,
                        SignCourseStatus = StudentCourseStatusEnum.NoSignUp,
                        CreateTime = item.CreateTime,
                        UpdateTime = item.UpdateTime,
                    };
                    if (signInfo != null)
                    {
                        model.SignCourseStatus = signInfo.SignCourseStatus;
                        model.IsPay = signInfo.IsPay;
                    }
                    if (model.TeacherCode == userCode)
                    {
                        //课程是自己创建的 
                        model.SignCourseStatus = StudentCourseStatusEnum.MyCreated;
                        model.IsPay = false;
                    }
                    if (item.CourseLabelCodes?.Count > 0)
                    {
                        foreach (var itemCode in item.CourseLabelCodes)
                        {
                            var labelName = allLabelList.FirstOrDefault(c => c.Code == itemCode)?.Name ?? string.Empty;
                            model.CourseLabels.Add(labelName);
                        }
                    }
                    list.Add(model);
                }
                res.TotalCount = dataList.TotalCount;
                res.PageData = list;
            }
            return res;
        }

        /// <summary>
        /// 查询课程专辑详情
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseAlbumExtendDTO> FindCourseAlbumByCodeAsync(FindCourseAlbumByCodeRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(request.LanguageCode))
            {
                request.LanguageCode = "en-US";
            }
            var result = await _courseDBService.FindCourseAlbumByCodeDBAsync(new FindCourseAlbumByCodeDBRequest
            {
                CourseAlbumCode = request.CourseAlbumCode,
                LanguageCode = request.LanguageCode
            });
            var data = new CourseAlbumExtendDTO();
            if (result != null)
            {
                var courseLabels = new List<string>();
                var allLabelList = await _educationDBService.GetAllCourseLabelDBAsync(new QueryCourseLabelDBRequest() { LanguageCode = request.LanguageCode }) ?? new List<CourseLabelDTO>();
                if (result.CourseLabelCodes?.Count > 0)
                {
                    foreach (var itemCode in result.CourseLabelCodes)
                    {
                        var labelName = allLabelList.FirstOrDefault(c => c.Code == itemCode)?.Name ?? string.Empty;
                        courseLabels.Add(labelName);
                    }
                }
                if (result.CourseInfos?.Count > 0)
                {
                    var payCourseCodes = result.CourseInfos.Where(v => v.Price > 0).Select(v => v.Code).ToList();
                    var signStudents = await _courseDBService.FindSignInListAsync(payCourseCodes);
                    var paySignStudents = signStudents.Where(v => v.StudentCode == userCode && v.IsPay).ToList();
                    foreach (var item in result.CourseInfos)
                    {
                        var paySignStudent = paySignStudents.FirstOrDefault(v => item.Code == v.CourseOrAlbumCode);
                        if (paySignStudent != null)
                        {
                            item.IsPay = true;
                        }
                    }
                }
                result.StudentCount = await _courseDBService.FindSignInListCountAsync(result.Code);
                data = new CourseAlbumExtendDTO()
                {
                    Code = result.Code,
                    Name = result.Name,
                    CourseInfos = result.CourseInfos,
                    SignInCount = result.StudentCount,
                    Cover = result.Cover,
                    CourseCodes = result.CourseCodes,
                    Introduction = result.Introduction,
                    CourseLabelCodes = result.CourseLabelCodes,
                    CourseLabelNames = courseLabels,
                    TeacherCode = result.TeacherCode,
                    TeacherName = result.TeacherName,
                    ViewRange = result.ViewRange,
                    Price = result.Price,
                    CreateTime = result.CreateTime,
                    Sort = result.Sort,
                    IsStick = result.IsStick,
                    SignCourseStatus = StudentCourseStatusEnum.NoSignUp
                };
                var signInfos = await _courseDBService.FindSignInListAsync(new List<string> { result.Code });
                var signInfo = signInfos.FirstOrDefault(x => x.StudentCode == userCode);
                if (signInfo != null)
                {
                    data.SignCourseStatus = signInfo.SignCourseStatus;
                    data.IsPay = signInfo.IsPay;
                }
                if (data.TeacherCode == userCode)
                {
                    //课程是自己创建的 
                    data.SignCourseStatus = StudentCourseStatusEnum.MyCreated;
                    data.IsPay = false;
                }
            }
            return data;
        }

        /// <summary>
        /// 查询学生所在班级列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<UserGroupDTO>> FindUserGroupByStudentAsync(QueryUserGroupPageRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.StudentCodeEmpty, "StudentCodeEmpty");
            }
            var dbReq = new FindUserGroupByStudentDBRequest
            {
                StudentCode = userCode,
                Keyword = request.Keyword,
                PageSize = request.PageSize,
                PageIndex = request.PageIndex
            };
            return await _courseDBService.FindUserGroupByStudentDBAsync(dbReq);
        }

        /// <summary>
        /// 查询老师创建的课程专辑列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<BaseCourseAlbumDTO>> FindCourseAlbumListAsync(TokenRequest request)
        {
            var dataList = new List<BaseCourseAlbumDTO>();
            var teacherCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(teacherCode))
            {
                ThrowRpcException(CustomerRpcCode.TeacherCodeEmpty, "Teacher code  is empty");
            }
            var list = await _courseDBService.GetCourseAlbumListByTeacherCodeDBAsync(teacherCode);
            if (list?.Count > 0)
            {
                dataList = list.Select(c => new BaseCourseAlbumDTO()
                {
                    Code = c.Code,
                    Name = c.Name
                }).ToList();
            }
            return dataList;
        }

        /// <summary>
        /// 老师添加学生，无需审核
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> AddUserGroupsStudentByTeacherAsync(TeacherAddStudentsInUserGroupsRequest request)
        {
            var dataList = new List<BaseCourseAlbumDTO>();
            var teacherCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(teacherCode))
            {
                ThrowRpcException(CustomerRpcCode.TeacherCodeEmpty, "Teacher code  is empty");
            }
            if (request.StudentCodes == null || request.StudentCodes.Count <= 0)
            {
                ThrowCustomerException(CustomerRpcCode.StudentCodeEmpty, "Student code is empty");
            }
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroup code is empty");
            }
            var userGroupInfo = await _educationDBService.FindUserGroupsByGroupCodeDBAsync(request.UserGroupCode, string.Empty);
            foreach (var studentCode in request.StudentCodes)
            {
                if (studentCode == teacherCode)
                {
                    //不合理，需要排除
                    ThrowRpcException(CustomerRpcCode.ClassTeacherCodeNotAddedClass, "Class teacher code not added class");
                }
                if (userGroupInfo.Students.Any(a => a.Code == studentCode))
                {
                    ThrowRpcException(CustomerRpcCode.StudentAlreadyIn, "Student already in");
                }
            }
            var req = new UserGroupRequest()
            {
                Code = request.UserGroupCode,
                Students = new List<StudentInfoDTO>()
            };
            req.Students = request.StudentCodes.Select(dx => new StudentInfoDTO()
            {
                Code = dx,
                LearnerStatus = LearnerStatusEnum.Accept
            }).ToList();
            var res = await _educationDBService.AddStudentsByGroupCodeDBAsync(req);
            return res;
        }

        /// <summary>
        /// 同意学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> JoinClassApprovedAsync(JoinClassApprovedRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.TeacherCodeEmpty, "TeacherCodeEmpty");
            }
            return await _educationDBService.JoinClassApprovedDBAsync(new JoinClassApprovedDBRequest
            {
                TeacherCode = userCode,
                UserGroupCode = request.UserGroupCode,
                Students = request.Students
            });
        }

        /// <summary>
        /// 拒绝学生加入班级
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> JoinClassRejectedAsync(JoinClassRejectedRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.TeacherCodeEmpty, "TeacherCodeEmpty");
            }
            return await _educationDBService.JoinClassRejectedDBAsync(new JoinClassRejectedDBRequest
            {
                TeacherCode = userCode,
                UserGroupCode = request.UserGroupCode,
                Students = request.Students
            });
        }

        /// <summary>
        /// 查询学生列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<UserGroupStudentsDTO>> FindStudentListAsync(FindStudentListRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrWhiteSpace(userCode))
            {
                ThrowCustomerException(CustomerRpcCode.TeacherCodeEmpty, "TeacherCodeEmpty");
            }
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroupCodeIsEmpty");
            }
            var dataList = await _educationDBService.FindStudentListDBAsync(new FindStudentListDBRequest
            {
                TeacherCode = userCode,
                UserGroupCode = request.UserGroupCode,
                LearnerStatus = request.LearnerStatus
            });
            return dataList;
        }

        /// <summary>
        /// 查询我的机构专家集合
        /// </summary>
        /// <param name="request">查询机构专家集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        public async Task<IList<UserBaseDTO>> FindMyOrganizationExpertsAsync(FindMyOrganizationExpertsRequest request)
        {
            var name = request.Name;
            var tokenDTO = await GetClientInfoByTokenAsync(request.Token);
            UserDTO userInfo = await _userDBService.FindUserByCodeAsync(tokenDTO.ClientId);
            List<UserBaseDTO> result = new List<UserBaseDTO>();
            var dbRequest = new GetUserListDBRequest
            {
                OrganizationCode = userInfo.RootOrganizationCode,
                OrganizationQueryType = OrganizationQueryTypeEnum.All,
                RankCode = "",
                PositionCode = "",
                RoleCodes = new List<string> { "Role_CertifiedExpert" },
                ExceptUsers = new List<string>()
            };
            var users = await _userDBService.GetUserListAsync(dbRequest);
            users = users.Where(x => x.FullName.Contains(name) || x.UserName.Contains(name)).ToList();
            if (users != null && users.Count > 0)
            {
                result = users.MappingTo<List<UserBaseDTO>>();
            }
            return result;
        }

        /// <summary>
        /// 转发进入课程通知到客户端（用于H5到壳子的通知）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SendCourseEntryNoticeAsync(SendCourseEntryNoticeRequest request)
        {
            var notification = new CourseEntryNotification()
            {
                NoticeData = request.NoticeData
            };
            //如客户端没有指定Id则发给当前用户
            if (string.IsNullOrWhiteSpace(request.ClientId))
            {
                request.ClientId = await GetClientIdByTokenAsync(request.Token);
            }
            var broadcastNotificationRequest = new BroadcastNotificationRequest
            {
                ClientIds = new List<string> { request.ClientId },
                Message = notification,
                JsonMessage = Newtonsoft.Json.JsonConvert.SerializeObject(notification),
                NotificationType = notification.NotificationType,
                TransactionType = TransactionTypeEnum.Education,
                ReceiverType = ApplicantTypeEnum.Client
            };
            return await _notificationService.BroadcastMessageAsync(broadcastNotificationRequest);
        }

        /// <summary>
        /// 根据课程code和用户code查询课程浏览记录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<CourseBrowsedRecordDTO> GetCourseBrowsedRecordByRelevanceCodeAsync(QueryBrowsedRecordRequest request)
        {
            var recordInfo = await _educationDBService.GetCourseBrowsedRecordByRelevanceCodeDBAsync(request);
            return recordInfo;
        }

        /// <summary>
        /// 保存课程浏览记录 
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<CourseBrowsedRecordDTO> SaveCourseBrowsedRecordAsync(CourseBrowsedRecordRequest request)
        {
            if (request.BrowsedUserType == BrowsedUserTypeEnum.Visitor)
            {
                if (string.IsNullOrEmpty(request.DisplayName))
                {
                    ThrowRpcException(CustomerRpcCode.CourseVisitorNameIsEmpty, "Course visitor name is empty");
                }
                if (string.IsNullOrEmpty(request.UserCode) && !string.IsNullOrEmpty(request.DisplayName))
                {
                    request.UserCode = Guid.NewGuid().ToString("N");
                }
            }
            var res = await _educationDBService.SaveCourseBrowsedRecordByDBAsync(request);            
            return res;
        }

        /// <summary>
        /// 获取课程观看人数和在线人数
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<CourseVisitorsStatisticResult> QueryCourseBrowsedCountAsync(QueryBrowsedRecordRequest request)
        {
            var courseDetailInfo = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = request.RelevanceCode });
            if (string.IsNullOrEmpty(courseDetailInfo?.Code))
            {
                return new CourseVisitorsStatisticResult();
            }
            var data = await _educationDBService.QueryCourseBrowsedCountByDBAsync(request);
            //查询课程老师等在线信息
            var userCodes = new List<string>() { courseDetailInfo.TeacherCode };
            if (courseDetailInfo.Experts?.Count > 0)
            {
                var expertUsers = courseDetailInfo.Experts.Select(c => c.Code).Distinct().ToList();
                userCodes.AddRange(expertUsers);
            }
            // if (courseDetailInfo.Assistants?.Count > 0)
            // {
            //     var assistantsUsers = courseDetailInfo.Assistants.Select(c => c.Code).Distinct().ToList();
            //     userCodes.AddRange(assistantsUsers);
            // }
            var memberRequest = new GetLiveMemberStatusInfosRequest()
            {
                UserCodes = userCodes
            };
            var stautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
            if (stautsInfo?.Count > 0)
            {
                foreach(var item in stautsInfo)
                {
                    var isOnline = (item.Status == LiveMemberStatus.Joined ? true : false);
                    if (isOnline)
                    {
                        data.OnlineCount++;
                    }
                }
            }
            return data;
        }

        /// <summary>
        /// 查询课程浏览记录
        /// </summary>
        /// <returns>是否成功</returns>
        public async Task<PageResult<CourseBrowsedRecordDTO>> QueryCourseBrowsedRecordsAsync(QueryBrowsedRecordPageRequest request)
        {
            var list = await _educationDBService.QueryCourseBrowsedRecordsByDBAsync(request);
            if (list?.TotalCount > 0 && list.PageData?.Count > 0) 
            {
                var userCodes = list.PageData.Where(c => c.BrowsedUserType != BrowsedUserTypeEnum.Visitor && c.BrowsedUserType != BrowsedUserTypeEnum.Student && c.BrowsedUserType != BrowsedUserTypeEnum.Assistant)?.Select(c => c.UserCode)?.ToList() ?? new List<string>();
                var memberRequest = new GetLiveMemberStatusInfosRequest()
                {
                    UserCodes = userCodes
                };
                var dataUserCodes = new List<string>();
                dataUserCodes.AddRange(userCodes);
                var tempCodes = list.PageData.Where(c => c.BrowsedUserType == BrowsedUserTypeEnum.Assistant)?.Select(c => c.UserCode)?.ToList() ?? new List<string>();
                if (tempCodes?.Count > 0)
                {
                    dataUserCodes.AddRange(tempCodes);
                }
                var userList = new List<UserDTO>();
                if (dataUserCodes?.Count > 0)
                {
                    dataUserCodes = dataUserCodes.Distinct().ToList();
                    userList = await _userDBService.FindUserListByUserCodesAsync(dataUserCodes);
                }
               
                var courseDetailInfo = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = request.RelevanceCode });
                if (courseDetailInfo == null)
                {
                    ThrowRpcException(CustomerRpcCode.LiveTeacherNotFound, "Live teacher not find");
                }
                
                if (courseDetailInfo.Status != CourseStatusEnum.Started)
                {
                    ThrowRpcException(CustomerRpcCode.CourseNotInProcess, "Course not in process");
                }
                var stautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
                if (stautsInfo?.Count > 0)
                {
                    var memberStautsInfo = stautsInfo?.Where(c => c.LiveRoomCode == request.RelevanceCode)?.ToList() ?? new List<WingInterfaceLibrary.DTO.LiveRoom.LiveMemberStatusDTO>();
                    foreach(var item in memberStautsInfo)
                    {
                        var cacheUserInfo = userList?.Where(c => c.UserCode == item.Code)?.FirstOrDefault() ?? new UserDTO();    
                        var findRecord = list.PageData.FirstOrDefault(c => c.UserCode == item.Code);
                        if (!string.IsNullOrEmpty(findRecord.UserCode))
                        {
                            if (item.Status == LiveMemberStatus.Joined || item.Status == LiveMemberStatus.Accepted)
                            {
                                findRecord.IsOnline = true;
                            }
                            else if (item.Code == courseDetailInfo.TeacherCode && courseDetailInfo.Status == CourseStatusEnum.Started)
                            {
                                findRecord.IsOnline = true;
                            }
                            else
                            {
                                findRecord.IsOnline = false;
                            }
                            findRecord.FromPlatform = item.LoginSource.ToString();
                            findRecord.Avatar = cacheUserInfo.HeadImageUrl;
                        }
                    }
                }

                memberRequest = new GetLiveMemberStatusInfosRequest()
                {
                    UserCodes = tempCodes
                };
                stautsInfo = await _rtcService.GetLiveMemberStatusInfosAsync(memberRequest);
                if (stautsInfo?.Count > 0)
                {
                    foreach(var item in stautsInfo)
                    {
                        var cacheUserInfo = userList?.Where(c => c.UserCode == item.Code)?.FirstOrDefault() ?? new UserDTO();    
                        var findRecord = list.PageData.FirstOrDefault(c => c.UserCode == item.Code);
                        if (!string.IsNullOrEmpty(findRecord.UserCode))
                        {
                            findRecord.FromPlatform = item.LoginSource.ToString();
                            findRecord.Avatar = cacheUserInfo.HeadImageUrl;
                        }
                    }
                }
            }
            return list;
        }

        /// <summary>
        /// 发送心跳，游客，观众
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns> 
        public async Task<bool> VisitorHeartRateAsync(QueryBrowsedRecordRequest request)
        {
            if (!string.IsNullOrEmpty(request.RelevanceCode))
            {
                if (string.IsNullOrEmpty(request.UserCode) && string.IsNullOrEmpty(request.DisplayName))
                {
                    return await Task.FromResult<bool>(true);
                }
                else 
                {
                    if (string.IsNullOrEmpty(request.UserCode))
                    {
                        request.UserCode = Guid.NewGuid().ToString("N");
                    }
                    _visitorLiveHeartRateManager.AddOrUpdate(request.UserCode, request.UserCode, request.DisplayName, request.RelevanceCode);
                    return await Task.FromResult<bool>(true);
                }
            }
            else
            {
                return await Task.FromResult<bool>(true);
            }
        }

        /// <summary>
        /// 游客，观众暂离直播间
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        public async Task<bool> VisitorLeaveLiveCourseAsync(QueryBrowsedRecordRequest request)
        {
            _visitorLiveHeartRateManager.Close(request.UserCode, request.UserCode, request.DisplayName, request.RelevanceCode);
            VisitorHandleClientLeave(request.RelevanceCode, request.UserCode, request.DisplayName);
            return await Task.FromResult<bool>(true);
        }

        /// <summary>
        /// 设置课程禁言
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> SetCourseLiveProhibitionAsync(CourseLiveProhibitionRequest request)
        {
            var res = await _courseDBService.SetCourseLiveProhibitionByDBAsync(request);
            return res;
        }

        #region private method  
        /// <summary>
        /// 加载在线游客到内存中
        /// </summary>
        private async void InitOnlineVisitorLiveHeartRate()
        {
            try
            {
                //获取所有的在线用户信息
                var recordsInfo = await _educationDBService.QueryAllOnlineVisitorInfoByDBAsync();

                //记录信息存在，添加到心跳中
                if (recordsInfo?.Count > 0)
                {
                    foreach(var item in recordsInfo)
                    {
                        _visitorLiveHeartRateManager.AddOrUpdate(item.UserCode, item.UserCode, item.DisplayName, item.RelevanceCode);
                    }
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService InitOnlineVisitorLiveHeartRate err, ex:{ex}");
            }
        }

        /// <summary>
        /// Visitor心跳检测进入房间
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void VisitorHandleClientJoined(string roomId, string clientId, string name)
        {
            try
            {
                //获取记录缓存
                var queryRequest = new QueryBrowsedRecordRequest()
                {
                    RelevanceCode = roomId,
                    UserCode = clientId,
                    DisplayName = name
                };
                var recordInfo = await _educationDBService.GetCourseBrowsedRecordByRelevanceCodeDBAsync(queryRequest);
                
                Logger.WriteLineInfo($"LiveService VisitorHandleClientJoined recordInfo: {JsonConvert.SerializeObject(recordInfo)}");

                var courseDetailInfo = await _courseDBService.FindCourseByCodeAsync(new FindCourseByCodeDBRequest() { Code = roomId });
                if (string.IsNullOrEmpty(courseDetailInfo?.Code) || courseDetailInfo.Status != CourseStatusEnum.Started)
                {
                    ThrowRpcException(CustomerRpcCode.CourseNotStart, "Course not start");
                }

                //存在, 且不在线，则更新状态
                if (!string.IsNullOrEmpty(recordInfo?.CourseBrowsedRecordCode) && !recordInfo.IsOnline)
                {
                    var updateEntity = recordInfo.MappingTo<CourseBrowsedRecordRequest>();
                    updateEntity.IsOnline = true;
                    updateEntity.IsHeartRateRequest = true;
                    var res = await _educationDBService.SaveCourseBrowsedRecordByDBAsync(updateEntity);                    
                }
                else if (string.IsNullOrEmpty(recordInfo?.CourseBrowsedRecordCode) && (!string.IsNullOrEmpty(clientId) || !string.IsNullOrEmpty(name)))
                {
                    
                    //不存在，新增
                    var insertEntity = new CourseBrowsedRecordRequest()
                    {
                        IsOnline = true,
                        RelevanceCode = roomId,
                        UserCode = clientId,
                        DisplayName = name,
                        BrowsedUserType = BrowsedUserTypeEnum.Visitor,
                        IsHeartRateRequest = true
                    };
                    var res = await _educationDBService.SaveCourseBrowsedRecordByDBAsync(insertEntity);           
                }
                else
                {
                    //存在，在线，则不处理
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService VisitorHandleClientJoined err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }

        /// <summary>
        /// Visitor心跳检测网络异常
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void VisitorHandleClientNetworkErr(string roomId, string clientId, string name)
        {
            try
            {
                //发送通知通知用户，网络异常，todo，暂时不做处理
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService VisitorHandleClientNetworkErr err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }

        /// <summary>
        /// Visitor心跳检测离线
        /// </summary>
        /// <param name="roomId"></param>
        /// <param name="clientId"></param>
        /// <param name="name"></param>
        /// <returns></returns> 
        private async void VisitorHandleClientLeave(string roomId, string clientId, string name)
        {
            try
            {
                //获取记录缓存
                var queryRequest = new QueryBrowsedRecordRequest()
                {
                    RelevanceCode = roomId,
                    UserCode = clientId,
                    DisplayName = name
                };
                var recordInfo = await _educationDBService.GetCourseBrowsedRecordByRelevanceCodeDBAsync(queryRequest);
                
                Logger.WriteLineInfo($"LiveService VisitorHandleClientLeave recordInfo: {JsonConvert.SerializeObject(recordInfo)}");
                //存在, 且在线，则更新为不在线
                if (!string.IsNullOrEmpty(recordInfo?.CourseBrowsedRecordCode) && recordInfo.IsOnline)
                {
                    var updateEntity = recordInfo.MappingTo<CourseBrowsedRecordRequest>();
                    updateEntity.IsOnline = false;
                    var res = await _educationDBService.VisitorBrowsedLeaveByDBAsync(updateEntity);                    
                }
                else
                {
                    //存在且不在线或者不存在，则不处理
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveService VisitorHandleClientLeave err, name:{name}, clientId:{clientId}, roomId:{roomId}, ex:{ex}");
            }
        }
        #endregion 


        /// <summary>
        /// 查询微信公众号轮播图列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<IList<WXCarouselsResult>> GetWXCarouselsAsync(GetWXCarouselRequest request)
        {
            var res = new List<WXCarouselsResult>();
            var dbReq = new GetAnnouncementsRequest()
            {
                Top = 100,
                AnnouncementType = request.AnnouncementType
            };
            var result = await _announcementDBService.GetAnnouncementsAsync(dbReq);
            if (result?.Count > 0)
            {
                foreach(var item in result)
                {
                    var model = new WXCarouselsResult();
                    model.Code = item.Code;
                    model.FileToken = item.FileToken;
                    if (item.LanguageConfigs?.Count > 0)
                    {                        
                        var content = item.LanguageConfigs.FirstOrDefault(c => c.Language == "zh-CN")?.Content ?? string.Empty;
                        model.TransferUrl = content;
                    }
                    res.Add(model);
                }
            }
            return res;
        }
    }
}
